@@ -1,6 +1,6 @@
 /*  Plzip - Parallel compressor compatible with lzip
     Copyright (C) 2009 Laszlo Ersek.
-    Copyright (C) 2009-2015 Antonio Diaz Diaz.
+    Copyright (C) 2009-2016 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -76,14 +76,14 @@ public:
       opacket_queues( workers ), num_working( workers ),
       num_workers( workers ), out_slots( slots ), slot_av( workers )
     {
-    xinit( &omutex ); xinit( &oav_or_exit );
-    for( unsigned i = 0; i < slot_av.size(); ++i ) xinit( &slot_av[i] );
+    xinit_mutex( &omutex ); xinit_cond( &oav_or_exit );
+    for( unsigned i = 0; i < slot_av.size(); ++i ) xinit_cond( &slot_av[i] );
     }
 
   ~Packet_courier()
     {
-    for( unsigned i = 0; i < slot_av.size(); ++i ) xdestroy( &slot_av[i] );
-    xdestroy( &oav_or_exit ); xdestroy( &omutex );
+    for( unsigned i = 0; i < slot_av.size(); ++i ) xdestroy_cond( &slot_av[i] );
+    xdestroy_cond( &oav_or_exit ); xdestroy_mutex( &omutex );
     }
 
   void worker_finished()
